#!/usr/bin/ruby
#
#   Check NRPE
#
# DESCRIPTION:
#   This is a simple NRPE check script for Sensu, We need to supply details
#   like Server, port, NRPE plugin, and plugin arguments
#
# OUTPUT:
#   plain text
#
# DEPENDENCIES:
#   gem: sensu-plugin
#   gem: nrpeclient
#
# USAGE:
#   check-nrpe -H host -c check_plugin -a 'plugin args'
#   check-nrpe -H host -c check_plugin -a 'plugin args' -m "(P|p)attern to match\.?"
#
# NOTES:
#   regex from https://github.com/naparuba/shinken/blob/master/shinken/misc/perfdata.py
#
# LICENSE:
#   Copyright (c) 2016 Scott Saunders <scosist@gmail.com>
#   Based on check-snmp.rb by Deepak Mohan Das   <deepakmdass88@gmail.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

require 'sensu-plugin/check/cli'
require 'nrpeclient'

# Class that checks the return from querying NRPE.
class CheckNRPE < Sensu::Plugin::Check::CLI
  option :host,
         short: '-H host',
         default: '127.0.0.1'

  option :check,
         short: '-c check_plugin',
         required: true

  option :args,
         short: '-a args',
         default: ''

  option :port,
         short: '-P port',
         description: 'port to use (default:5666)',
         default: 5666,
         proc: proc(&:to_i)

  option :disable_ssl,
         short: '-d',
         description: 'disable ssl (default:false)',
         boolean: true,
         default: false

  option :match,
         short: '-m match',
         description: 'Regex pattern to match against returned buffer'

  def run
    begin
      request = Nrpeclient::CheckNrpe.new(host: config[:host].to_s, port: config[:port], ssl: !config[:disable_ssl])
      response = request.send_command(config[:check].to_s, config[:args].to_s)
    rescue Errno::ETIMEDOUT
      unknown "#{config[:host]} not responding"
    rescue => e
      unknown "An unknown error occured: #{e.inspect}"
    end

    if config[:match]
      if response.buffer.to_s =~ /#{config[:match]}/
        ok
      else
        critical "Buffer: #{response.buffer} failed to match Pattern: #{config[:match]}"
      end
    else
      response_captures = response.buffer.scan(/[A-Z ]*?([A-Z]+)\s?[:-]{1}\s(.*)/)
      response_status = response_captures[0][0].downcase
      response_data = response_captures[0][1]
      case response_status
      when 'critical'
        critical response_data
      when 'warning'
        warning response_data
      else
        ok response_data
      end
    end
  end
end
